@@ -1002,6 +1002,13 @@ int32 kbd_inkey(int32 arg) {
       return 0;
     }
 
+#ifdef SDL12_COMPAT_HEADERS
+    /* Really dirty hack for SDL12-compat not quite able to make its mind up over whether it
+       should be using SDLK_PRINT or SDLK_SYSREQ when they're both on the same key. Basically,
+       if it's lit the flag for SYSREQ, we'll manually light the PRINT flag too. Ugh. */
+    if (keystate[SDLK_SYSREQ]) keystate[SDLK_PRINT]=keystate[SDLK_SYSREQ];
+#endif
+
     if (arg < 0x080) {				/* Test for single keypress, INKEY-key	*/
       switch (arg) {
 #ifdef TARGET_DOSWIN
@@ -1411,12 +1418,14 @@ static int32 keyboard;          /* File descriptor for keyboard */
 void push_key(int32 ch) {
   holdcount++;
   holdstack[holdcount] = ch;
+  //fprintf(stderr, "push_key called, ch=%d, holdcount=%d\n", ch, holdcount);
 }
 
 /*
 ** pop_key - Remove a key from the held key stack
 */
 static int32 pop_key(void) {
+  //fprintf(stderr, "pop_key called, ch=%d, holdcount=%d\n", holdstack[holdcount], holdcount-1);
   return holdstack[holdcount--];
 }
 
@@ -1581,7 +1590,7 @@ static boolean waitkey(int wait) {
 #endif
 }
 
-#ifdef USE_SDL
+#ifdef SDL12_COMPAT_HEADERS
 /* sdl12-compat doesn't return sensible values for CTRL+key in keysym.unicode.
    This is a translation table to provide the same outputs as SDL-1.2.15 */
 uint32 compatkeyfix(SDLKey sym, SDLMod mod) {
@@ -1704,6 +1713,9 @@ int32 read_key(void) {
             case SDLK_F11: case SDLK_F12:
               ch = 0x81 + ev.key.keysym.sym - SDLK_F1;
               break;
+#ifdef SDL12_COMPAT_HEADERS
+            case SDLK_SYSREQ:   ch=0x80; break; /* sdl12-compat sometimes uses this for some reason */
+#endif
             case SDLK_PRINT:    ch=0x80; break;
             case SDLK_PAUSE:    ch=0xC4; break;
             case SDLK_INSERT:   ch=0xC6; break;
@@ -1721,8 +1733,10 @@ int32 read_key(void) {
 //            return ESCAPE;
             default:
               ch = ev.key.keysym.unicode;
+#ifdef SDL12_COMPAT_HEADERS
               /* Workaround an sdl12-compat bug */
-              if ((ch == 0) && (ev.key.keysym.mod & KMOD_CTRL)) ch=compatkeyfix(ev.key.keysym.sym, ev.key.keysym.mod);
+              if ((ch >= 32) && (ev.key.keysym.mod & KMOD_CTRL)) ch=compatkeyfix(ev.key.keysym.sym, ev.key.keysym.mod);
+#endif
               //fprintf(stderr, "keysym.unicode=%d keysym.sym=%d keysym.mod=0x%X\n", ch, ev.key.keysym.sym, ev.key.keysym.mod);
               if (ch < 0x100) {
                 matrixflags.noupdate = 0;
