@@ -70,6 +70,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <dirent.h>
 //#include <stdarg.h>
 #include <ctype.h>
 #include <time.h>
@@ -101,7 +102,9 @@
 #if defined(TARGET_MINGW) || defined(TARGET_UNIX) || defined(TARGET_MACOSX)
 #include <sys/time.h>
 #include <sys/types.h>
+#include <fcntl.h>
 #include <unistd.h>
+#include <sys/stat.h>
 #endif
 
 #ifdef USE_SDL
@@ -148,8 +151,8 @@ static void native_oscli(char *command, char *respfile, FILE *respfh);
 #define GPPUDCLK0 38
 #define GPPUDCLK1 39
 
-#define PI_BANK (inregs[0]>>5)
-#define PI_BIT  (1<<(inregs[0]&0x1F))
+#define PI_BANK (inregs[0].i >> 5)
+#define PI_BIT  (1<<(inregs[0].i & 0x1F))
 
 #define PI_INPUT  0
 #define PI_OUTPUT 1
@@ -1132,7 +1135,7 @@ void mos_waitdelay(int32 time) {
   tbase=mos_centiseconds();
   while(mos_centiseconds() < tbase + time) {
 #ifdef USE_SDL
-    if (kbd_escpoll()) {
+    if (basicvars.escape) {
       time=0;
       error(ERR_ESCAPE);
     }
@@ -1250,39 +1253,40 @@ static unsigned int cmd_parse_num(char** text)
 /*
  * List of *commands implemented by this code
  */
-#define CMD_UNKNOWN		0
-#define CMD_KEY			1
-#define CMD_CAT			2
-#define CMD_CD			3
-#define CMD_QUIT		4
-#define CMD_WINDOW		6
-#define CMD_FX			7
-#define CMD_VER			8
-#define CMD_TITLE		9
-#define CMD_HELP		10
-#define CMD_WINTITLE		11
-#define CMD_FULLSCREEN		12
-#define CMD_NEWMODE		13
-#define CMD_REFRESH		14
-#define CMD_SCREENSAVE		15
-#define CMD_SCREENLOAD		16
-#define CMD_SHOW		17
-#define CMD_EXEC		18
-#define CMD_SPOOL		19
-#define CMD_SPOOLON		20
-#define CMD_LOAD		27
-#define CMD_SAVE		28
-#define CMD_VOLUME		29
-#define CMD_CHANNELVOICE	30
-#define CMD_VOICES		31
-#define CMD_POINTER		32
-#define CMD_BRANDYINFO		33
-#define HELP_BASIC		1024
-#define HELP_HOST		1025
-#define HELP_MOS		1026
-#define HELP_MATRIX		1027
-#define HELP_SOUND		1028
-#define HELP_MEMINFO		1029
+#define CMD_UNKNOWN       0
+#define CMD_KEY           1
+#define CMD_CAT           2
+#define CMD_EX            3
+#define CMD_CD            4
+#define CMD_QUIT          5
+#define CMD_WINDOW        6
+#define CMD_FX            7
+#define CMD_VER           8
+#define CMD_TITLE         9
+#define CMD_HELP          10
+#define CMD_WINTITLE      11
+#define CMD_FULLSCREEN    12
+#define CMD_NEWMODE       13
+#define CMD_REFRESH       14
+#define CMD_SCREENSAVE    15
+#define CMD_SCREENLOAD    16
+#define CMD_SHOW          17
+#define CMD_EXEC          18
+#define CMD_SPOOL         19
+#define CMD_SPOOLON       20
+#define CMD_LOAD          27
+#define CMD_SAVE          28
+#define CMD_VOLUME        29
+#define CMD_CHANNELVOICE  30
+#define CMD_VOICES        31
+#define CMD_POINTER       32
+#define CMD_BRANDYINFO    33
+#define HELP_BASIC        1024
+#define HELP_HOST         1025
+#define HELP_MOS          1026
+#define HELP_MATRIX       1027
+#define HELP_SOUND        1028
+#define HELP_MEMINFO      1029
 
 #define CMDTABSIZE 256
 
@@ -1358,6 +1362,7 @@ void make_cmdtab(){
   add_cmd( "cat",          CMD_CAT  );
   add_cmd( "cd",           CMD_CD   );
   add_cmd( "chdir",        CMD_CD   );
+  add_cmd( "ex",           CMD_EX   );
   add_cmd( "quit",         CMD_QUIT );
   add_cmd( "fx",           CMD_FX   );
   add_cmd( "help",         CMD_HELP );
@@ -1391,20 +1396,66 @@ void make_cmdtab(){
 }
 
 /*
- * *.(<directory>)
+ * *.(<directory>) or *cat (<directory>)
  * Catalogue directory
- * NB, Only *. does catalog, *CAT passed to host OS
+ * Internal implementation, slightly styled on BBC Micro output, with
+ * column spacing to work in any 20/40/80 screen mode.
  */
 static void cmd_cat(char *command) {
-	while (*command == ' ') command++;	// Skip spaces
-#if defined(TARGET_DOSWIN)
-	native_oscli("dir /w", NIL, NULL);
-#elif defined(TARGET_MACOSX) | defined(TARGET_UNIX)
-	native_oscli("ls -l", NIL, NULL);
-	// native_oscli("ls -C", NIL, NULL);
-#elif defined(TARGET_AMIGA)
-	native_oscli("list", NIL, NULL);
+  char dbuf[FILENAME_MAX+1];
+  char fbuf[FILENAME_MAX+1];
+  int buflen=0, loop=0;
+  struct dirent *entry;
+  DIR *dirp;
+  struct stat statbuf;
+
+  memset(dbuf,0,FILENAME_MAX+1);
+  memset(fbuf,0,FILENAME_MAX+1);
+  getcwd(dbuf, FILENAME_MAX);
+  buflen=FILENAME_MAX - strlen(dbuf);
+  while (*command && (*command != ' ')) command++;	// Skip command
+	while (*command && (*command == ' ')) command++;	// Skip spaces
+  if (strlen(command)) {
+#ifdef TARGET_MINGW
+    if ((*(command+1) == ':') && ((*(command+2) == '\\') || (*(command+2) == '/'))) {
+#else
+    if (*command == '/') {
 #endif
+      strncpy(dbuf, command, buflen);
+    } else {
+      buflen--;
+      strncat(dbuf, "/", buflen);
+      strncat(dbuf, command, buflen-strlen(command));
+    }
+  }
+  emulate_printf("Dir. %s\r\n", dbuf);
+  dirp=opendir(dbuf);
+  if (!dirp) error(ERR_DIRNOTFOUND);
+  else {
+    emulate_printf("\r\n", dbuf);
+    while ((entry = readdir(dirp)) != NULL) {
+      if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) continue;
+      strncpy(fbuf, dbuf, FILENAME_MAX+1);
+      strncat(fbuf, "/", 2);
+      strncat(fbuf, entry->d_name, FILENAME_MAX - strlen(dbuf));
+      stat(fbuf, &statbuf);
+      emulate_printf("%s", entry->d_name);
+      loop=(1000 - (5+strlen(entry->d_name))) % 20;
+      while(loop--) emulate_printf(" ");
+      emulate_printf("/");
+      if ((statbuf.st_mode & S_IFMT) == S_IFDIR) emulate_printf("D");
+      if (!access(fbuf, W_OK))
+        emulate_printf("W");
+      else
+        emulate_printf("L");
+      if (!access(fbuf, R_OK))
+        emulate_printf("R ");
+      else
+        emulate_printf("  ");
+      if ((statbuf.st_mode & S_IFMT) != S_IFDIR) emulate_printf(" ");
+    }
+  }
+  emulate_printf("\r\n");
 }
 
 /* Sets the window title. */
@@ -1418,6 +1469,19 @@ static void cmd_wintitle(char *command) {
   return;
 }
 
+static void cmd_ex(char *command) {
+	while (*command == ' ') command++;	// Skip spaces
+#if defined(TARGET_DOSWIN)
+	native_oscli("dir", NIL, NULL);
+#elif defined(TARGET_MACOSX) | defined(TARGET_UNIX)
+	native_oscli("ls -l", NIL, NULL);
+	// native_oscli("ls -C", NIL, NULL);
+#elif defined(TARGET_AMIGA)
+	native_oscli("list", NIL, NULL);
+#endif
+
+}
+
 static void cmd_fullscreen(char *command) {
 #ifdef USE_SDL
   int flag=3;
@@ -2043,32 +2107,33 @@ void mos_oscli(char *command, char *respfile, FILE *respfh) {
  */
     cmd = check_command(command);
     switch(cmd){
-      case CMD_KEY:		cmd_key(command+3); return;
-      case CMD_CAT:		cmd_cat(command); return;
-      case CMD_QUIT:		cmd_quit(command+4); return;
-      case CMD_HELP:		cmd_help(command+4); return;
-      case CMD_CD:		cmd_cd(command+2); return;
-      case CMD_FX:		cmd_fx(command+2); return;
-      case CMD_SHOW:		cmd_show(command+4); return;
-      case CMD_EXEC:		cmd_exec(command+4); return;
-      case CMD_SPOOL:		cmd_spool(command+5,0); return;
-      case CMD_SPOOLON:		cmd_spool(command+7,1); return;
-//    case CMD_VER:		cmd_ver(); return;
-      case CMD_SCREENSAVE:	cmd_screensave(command+10); return;
-      case CMD_SCREENLOAD:	cmd_screenload(command+10); return;
-      case CMD_WINTITLE:	cmd_wintitle(command+8); return;
-      case CMD_FULLSCREEN:	cmd_fullscreen(command+10); return;
-      case CMD_NEWMODE:		cmd_newmode(command+7); return;
-      case CMD_REFRESH:		cmd_refresh(command+7); return;
-      case CMD_BRANDYINFO:	cmd_brandyinfo(); return;
+      case CMD_KEY:           cmd_key(command+3); return;
+      case CMD_CAT:           cmd_cat(command); return;
+      case CMD_EX:            cmd_ex(command); return;
+      case CMD_QUIT:          cmd_quit(command+4); return;
+      case CMD_HELP:          cmd_help(command+4); return;
+      case CMD_CD:            cmd_cd(command+2); return;
+      case CMD_FX:            cmd_fx(command+2); return;
+      case CMD_SHOW:          cmd_show(command+4); return;
+      case CMD_EXEC:          cmd_exec(command+4); return;
+      case CMD_SPOOL:         cmd_spool(command+5,0); return;
+      case CMD_SPOOLON:       cmd_spool(command+7,1); return;
+//    case CMD_VER:           cmd_ver(); return;
+      case CMD_SCREENSAVE:    cmd_screensave(command+10); return;
+      case CMD_SCREENLOAD:    cmd_screenload(command+10); return;
+      case CMD_WINTITLE:      cmd_wintitle(command+8); return;
+      case CMD_FULLSCREEN:    cmd_fullscreen(command+10); return;
+      case CMD_NEWMODE:       cmd_newmode(command+7); return;
+      case CMD_REFRESH:       cmd_refresh(command+7); return;
+      case CMD_BRANDYINFO:    cmd_brandyinfo(); return;
 
-      case CMD_LOAD:		cmd_load(command+4); return;
-      case CMD_SAVE:		cmd_save(command+4); return;
+      case CMD_LOAD:          cmd_load(command+4); return;
+      case CMD_SAVE:          cmd_save(command+4); return;
 
-      case CMD_VOLUME:		cmd_volume(command+6);return;
-      case CMD_CHANNELVOICE:	cmd_channelvoice(command+12);return;
-      case CMD_VOICES:		cmd_voices();return;
-      case CMD_POINTER:		cmd_pointer(command+7);return;
+      case CMD_VOLUME:        cmd_volume(command+6);return;
+      case CMD_CHANNELVOICE:  cmd_channelvoice(command+12);return;
+      case CMD_VOICES:        cmd_voices();return;
+      case CMD_POINTER:       cmd_pointer(command+7);return;
     }
   }
 
@@ -2748,8 +2813,7 @@ static byte _sysvar[] = { 0, 0, 0, 0, 0, 0, 0,  0, 0, 0,   /* &A6 - &AF */
 0 }; /* Overflow for &FF+1 */
 byte *sysvar = _sysvar-166;
 
-static int32 mos_osbyte(int32 areg, int32 xreg, int32 yreg, int32 xflag)
-{
+static int32 mos_osbyte(int32 areg, int32 xreg, int32 yreg, int32 xflag){
 int tmp,new;
 
 tmp=(areg=areg & 0xFF);	// Prevent any sillyness
@@ -2760,6 +2824,7 @@ if (areg>=166) {
   yreg = sysvar[areg+1] & 0xFF;
   sysvar[areg] = new;
 // Some variables are 'unclean' because we don't control the kernel
+  if (areg==210) { if (sysvar[210]!=0) mos_sound_off(); else mos_sound_on(); }
   if (areg==220) kbd_escchar(new, xreg);
   return (0 << 30) | (yreg << 16) | (xreg << 8) | areg;
 }
@@ -2855,6 +2920,9 @@ switch (areg) {
 		else			return ((areg & 0xFF0000) >> 16) | ((areg & 0xFFFF) << 8);
 //	case 133:			// OSBYTE 133 - Read screen start for MODE - not implemented in RISC OS.
 
+	case 138:
+		if (xreg==0) push_key(yreg);
+		return ((kbd_inkey(xreg | yreg<<8) & 0xFFFF) << 8) | 0x8A; 
 	case 160:			// OSBYTE 160 - Read VDU variable
 		return emulate_vdufn(xreg) << 8 | 160;
 #ifdef USE_SDL
@@ -2872,12 +2940,11 @@ switch (areg) {
 		  osbyte113(1);
 		  emulate_vdu(6);
 		}
-    if (xreg==242) { // GXR and dot pattern
-      return (osbyte163_242(yreg) << 8);
-    }
+		if (xreg==242) { // GXR and dot pattern
+		  return (osbyte163_242(yreg) << 8);
+		}
 		break;
 #endif
-
 // This is now in keyboard.c
 //	case 200:		// OSBYTE 200 - bit 0 disables escape if unset
 //	case 229:		// OSBYTE 229 - Enable or disable escape
