@@ -39,9 +39,12 @@
 #include <errno.h>
 #ifdef TARGET_RISCOS
 #include <ctype.h>
+#ifdef __TARGET_SCL__
+#include <sys/ioctl.h>
+#endif /* __TARGET_SCL__ */
 #include "kernel.h"
 #include "swis.h"
-#endif
+#endif /* TARGET_RISCOS */
 
 #include "basicdefs.h"
 #include "errors.h"
@@ -174,10 +177,13 @@ void brandynet_init() {
 int brandynet_connect(char *dest, char type) {
 #if defined(TARGET_RISCOS) | defined(MINIX_OLDNET)
   char *host, *port;
-  int n, mysocket, portnum, result;
+  int n, mysocket, portnum, result, flags;
   struct sockaddr_in netdest;
   struct hostent *he = NULL;
   struct in_addr *inaddr = NULL;
+#ifdef __TARGET_SCL__
+  unsigned long opt;
+#endif
 
   if(matrixflags.networking==0) {
     error(ERR_NET_NOTSUPP);
@@ -225,16 +231,20 @@ int brandynet_connect(char *dest, char type) {
   }
   free(inaddr);                              /* Don't need this any more */
 
-#ifndef __TARGET_SCL__
-  /* SharedCLibrary doesn't support this */
-  fcntl(mysocket, F_SETFL, O_NONBLOCK);
+#ifdef __TARGET_SCL__
+  /* SharedCLibrary doesn't support fcntl */
+  opt=1;
+  socketioctl(mysocket, FIONBIO, &opt);
+#else
+  flags = fcntl(mysocket, F_GETFL, 0);
+  fcntl(mysocket, F_SETFL, flags | O_NONBLOCK);
 #endif
   netsockets[n] = mysocket;
   return(n);
 
 #else /* not TARGET_RISCOS */
   char *host, *port;
-  int n, mysocket=0, ret;
+  int n, mysocket=0, ret, flags;
   struct addrinfo hints, *addrdata, *rp;
 #ifdef TARGET_MINGW
   unsigned long opt;
@@ -296,7 +306,8 @@ int brandynet_connect(char *dest, char type) {
   opt=1;
   ioctlsocket(mysocket, FIONBIO, &opt);
 #else
-  fcntl(mysocket, F_SETFL, O_NONBLOCK);
+  flags = fcntl(mysocket, F_GETFL, 0);
+  fcntl(mysocket, F_SETFL, flags | O_NONBLOCK);
 #endif
   netsockets[n] = mysocket;
   return(n);
