@@ -35,8 +35,8 @@
 #define BRANDY_NAME  "Matrix"
 #define BRANDY_MAJOR "1"
 #define BRANDY_MINOR "22"
-#define BRANDY_PATCHLEVEL "14"
-#define BRANDY_DATE       "31 Dec 2022"
+#define BRANDY_PATCHLEVEL "15"
+#define BRANDY_DATE       "21 Mar 2023"
 // #define BRANDY_PATCHDATE  "JGH191007"
 
 /*
@@ -77,13 +77,14 @@
 **
 ** The following are suitable for ARM and X86
 */
-typedef unsigned char uint8;		/* Type for unsigned 8-bit integer variables in Basic */
-typedef int int32;			/* Type for 32-bit integer variables in Basic */
-typedef unsigned int uint32;		/* 32-bit unsigned integer */
-typedef double float64;			/* Type for 64-bit floating point variables in Basic */
-typedef long long int int64;		/* Type for 64-bit integer variables */
-typedef unsigned long long int uint64;	/* 64-bit unsigned integer */
-
+typedef unsigned char uint8;            /* Type for unsigned 8-bit integer variables in Basic */
+typedef int int32;                      /* Type for 32-bit integer variables in Basic */
+typedef unsigned int uint32;            /* 32-bit unsigned integer */
+typedef double float64;                 /* Type for 64-bit floating point variables in Basic */
+typedef long long int int64;            /* Type for 64-bit integer variables */
+typedef unsigned long long int uint64;  /* 64-bit unsigned integer */
+typedef long int nativeint;             /* 32 or 64-bit depending on architecture */
+typedef unsigned long int nativeuint;   /* 32 or 64-bit depending on architecture */
 
 /*
 ** The following macros define the OS under which the program is being
